@@ -12,7 +12,7 @@ use Kvasir::Util qw(is_existing_package);
 
 use Object::Tiny qw(_ruleset);
 
-our $VERSION = "0.02";
+our $VERSION = "0.03";
 
 sub _new {
     my ($pkg) = @_;
@@ -45,10 +45,11 @@ sub load_string {
 {
     my %Node_Handler = (
         action      => "_process_action",
+        defaults    => "_process_defaults",
         input       => "_process_input",
         output      => "_process_output",
-        prehook     => "_process_prehook",
         posthook    => "_process_posthook",
+        prehook     => "_process_prehook",
         rule        => "_process_rule",
         ruleset     => "_process_ruleset",
         run         => "_process_run",
@@ -85,42 +86,60 @@ sub load_string {
 
 sub _process_action {
     my ($self, $action, $engine) = @_;
-    my ($name, $class, @args) = $self->_process_std_element($action);
-    $engine->add_action($name => $class, @args);        
+    my ($name, $class, $defaults, @args) = $self->_process_std_element($action);
+    $engine->add_action($name => $class, $defaults, @args);        
 };
 
 sub _process_input {
     my ($self, $input, $engine) = @_;
-    my ($name, $class, @args) = $self->_process_std_element($input);
-    $engine->add_input($name => $class, @args);
+    my ($name, $class, $defaults, @args) = $self->_process_std_element($input);
+    $engine->add_input($name => $class, $defaults, @args);
 };
 
 sub _process_output {
     my ($self, $output, $engine) = @_;
-    my ($name, $class, @args) = $self->_process_std_element($output);
-    $engine->add_output($name => $class, @args);
+    my ($name, $class, $defaults, @args) = $self->_process_std_element($output);
+    $engine->add_output($name => $class, $defaults, @args);
 };
 
 sub _process_prehook { 
     my ($self, $hook, $engine) = @_;
-    my ($name, $class, @args) = $self->_process_std_element($hook);
-    $engine->add_hook($name => $class, @args);
+    my ($name, $class, $defaults, @args) = $self->_process_std_element($hook);
+    $engine->add_hook($name => $class, $defaults, @args);
     $engine->add_pre_hook($name);
 };
 
 sub _process_posthook { 
     my ($self, $hook, $engine) = @_;
-    my ($name, $class, @args) = $self->_process_std_element($hook);
-    $engine->add_hook($name => $class, @args);
+    my ($name, $class, $defaults, @args) = $self->_process_std_element($hook);
+    $engine->add_hook($name => $class, $defaults, @args);
     $engine->add_post_hook($name);
 };
 
 sub _process_rule {
     my ($self, $rule, $engine) = @_;
-    my ($name, $class, @args) = $self->_process_std_element($rule);
-    $engine->add_rule($name => $class, @args);        
+    my ($name, $class, $defaults, @args) = $self->_process_std_element($rule);
+    $engine->add_rule($name => $class, $defaults, @args);        
 };
 
+sub _process_defaults {
+    my ($self, $defaults, $engine) = @_;
+    
+    my $name = $defaults->getAttribute("name");
+
+    my @args;
+    for my $arg ($defaults->childNodes) {
+        next unless $arg->isa("XML::LibXML::Element");
+        my $name = $arg->nodeName;
+        my $value = $arg->hasChildNodes ? $arg->textContent : undef;
+        push @args, $name => $value;
+    }
+    
+    my $data = { @args };
+    
+    $engine->add_defaults($name, $data);
+}
+
 sub _process_ruleset {
     my ($self, $ruleset, $engine) = @_;
     
@@ -185,9 +204,13 @@ sub _process_std_element {
     }
     my $class = $element->getAttribute("instanceOf");
     
+    my $defaults = $element->getAttribute("defaults");
+    $defaults = "" if !defined $defaults;
+    $defaults = [split/,\s*|\s+/, $defaults];
+
     my @args = $self->_process_args($element, $class);    
 
-    return ($name, $class, @args);
+    return ($name, $class, $defaults, @args);
 }
 
 sub _process_rules {
@@ -233,15 +256,15 @@ sub _process_args {
         return $class->process_xml_loader_args($element);
     }
     
-    my %args;
+    my @args;
     for my $arg ($element->childNodes) {
         next unless $arg->isa("XML::LibXML::Element");
         my $name = $arg->nodeName;
         my $value = $arg->hasChildNodes ? $arg->textContent : undef;
-        $args{$name} = $value;
+        push @args, $name => $value;
     }
     
-    return (\%args);
+    return @args;
 }
 
 1;
@@ -293,6 +316,10 @@ B<< <action> >> - Declares an action.
 
 =item *
 
+B<< <defaults> >> - Defines a default argument set
+
+=item *
+
 B<< <input> >> - Declares an input.
 
 =item *
@@ -328,6 +355,11 @@ The name of the entity to define in the engine.
 
 The class that implements the entity and that'll be instansiated when the engine is runned.
 
+=item defaults
+
+The default arguments to the entity as defined by a previously declared B<< <defaults> >>. 
+Separate multiple defaults with comma and/or whitespace.
+
 =back
 
 If the class defined by I<instanceOf> implements the method C<process_xml_loader_args> it will be 
@@ -340,6 +372,12 @@ where the elements name is the key and its text content its value. If a child is
 that is has no children (as in C<< <foo/> >>) its value will be undef. This hash will be passed 
 to the constructor as a hash reference.
 
+=head2 Defaults
+
+By using the tag B<< <defaults> >> t is possible to declare common arguments that can be 
+reused by multiple entities. Its children will be interpreted as key/value pairs. The required 
+attribute 'name' defines the name for the defaults.
+
 =head2 Rulesets
 
 By using the tag B<< <ruleset> >> it is possible to give a set of rules a shared name that can later be 
